package com.ptplib.usbcamera.test;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.ListView;

import com.ptplib.usbcamera.BaselineInitiator;
import com.ptplib.usbcamera.PTPException;
import com.ptplib.usbcamera.eos.EosInitiator;
import com.ptplib.usbcamera.nikon.NikonInitiator;
import com.strickling.usbcamera.R;

public class MyTestActivity extends Activity {

    private ImageView photoIv;
    private ListView infoListView;
    private MyTestAdapter adapter;

    private UsbManager mUsbManager;
    private BaselineInitiator bi;

    private static final String TAG = "USBTest";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_my_test);

        mUsbManager = (UsbManager)getSystemService(Context.USB_SERVICE);

        photoIv = (ImageView)findViewById(R.id.iv_latest_photo);
        infoListView = (ListView)findViewById(R.id.list_info);
        adapter = new MyTestAdapter(this);
        infoListView.setAdapter(adapter);

        findViewById(R.id.btn_get_devece_info).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });

        findViewById(R.id.btn_get_photo).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        registerUSBReceiver();
        initDevice (searchDevice ());
    }

    @Override
    protected void onPause() {
        super.onPause();
        unregisterReceiver(mUsbReceiver);
        detachDevice();
    }

    BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            UsbDevice device = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
            if (UsbManager.ACTION_USB_DEVICE_ATTACHED.equals(action)) {
                adapter.addInfo("监听到有USB设备插入 "+ device.getDeviceName());
                initDevice (device);
            } else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
                adapter.addInfo("监听到有USB设备卸载 "+ device.getDeviceName());
                if (bi != null && bi.getSession() != null) bi.getSession().close();
                detachDevice ();
            }
        }
    };

    // search connected devices, returns only protocol 0 devices
    public UsbDevice searchDevice () {
        UsbDevice device = null;
        adapter.addInfo("开始搜素USB设备---------- ");
        for (UsbDevice lDevice :  mUsbManager.getDeviceList().values()) {
            adapter.addInfo("搜素到USB设备 "+ lDevice.getDeviceName() + " 支持协议为"+ lDevice.getDeviceProtocol());
            if (lDevice.getDeviceProtocol() == 0) {
                device = lDevice;
                adapter.addInfo("搜素到支持PTP的USB设备 "+ lDevice.getDeviceName() + " 支持协议为"+ lDevice.getDeviceProtocol());
            }
        }
        adapter.addInfo("搜素USB设备完成---------- ");
        return device;
    }

    public void initDevice (UsbDevice device) {
        if (device != null){
            adapter.addInfo("初始化设备 "+ device.getDeviceName());
            try {
                bi = new BaselineInitiator (device, mUsbManager.openDevice(device));
                // Select appropriate deviceInitiator, VIDs see http://www.linux-usb.org/usb.ids
                if (bi.getDevice().getVendorId() == EosInitiator.CANON_VID) {
                    try {
                        bi.getClearStatus();
                        bi.close();
                    } catch (PTPException e) {e.printStackTrace();
                        adapter.addInfo("初始化设备异常 "+ e);
                    }
                    adapter.addInfo("初始化设备成功， 是Cannon");
                    bi = new EosInitiator (device, mUsbManager.openDevice(device));
                }
                else if (device.getVendorId() == NikonInitiator.NIKON_VID) {
                    try {
                        bi.getClearStatus();
                        bi.close();
                    } catch (PTPException e) {e.printStackTrace();
                        adapter.addInfo("初始化设备异常 "+ e);
                    }
                    adapter.addInfo("初始化设备成功， 是Nikon");
                    bi = new NikonInitiator (device, mUsbManager.openDevice(device));
                }
                adapter.addInfo("开始建立session");
                bi.openSession();
            } catch (PTPException e) {
                e.printStackTrace();
                adapter.addInfo("初始化设备发生错误 "+ e);
            }
        }
    }

    public void detachDevice () {
        if (bi != null) {
            if (bi.getDevice() != null) Log.d(TAG, "detachDevice: " +bi.getDevice().getDeviceName());
            try {
                bi.close();
            } catch (PTPException e) {
                e.printStackTrace();
            }
        }
    }
    private void registerUSBReceiver(){
        IntentFilter filter = new IntentFilter();
        filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
        registerReceiver(mUsbReceiver, filter);
    }
}
